@@ -2,22 +2,22 @@
 
 use std::collections::HashMap;
 
-use authentic::credential::{HttpRealmCredential, UsernamePasswordCredential};
+use authentic::credential::{HttpRealmCredentials, UsernamePasswordCredential};
 use authentic::hyper::{BasicAuthentication, HttpAuthentication};
 use authentic::{AuthenticationScheme, AuthenticationStep, WithAuthentication};
 use http::StatusCode;
 use hyper::Client;
 use hyper_tls::HttpsConnector;
 
-/// Test direct basic authentication, passing the username and password on the first request.
+/// Direct basic authentication, passing the username and password on the first request.
 #[::tokio::test]
 async fn test_basic_authentication(
 ) -> Result<(), Box<dyn std::error::Error + Send + Sync + 'static>> {
     let https = HttpsConnector::new();
     let client = Client::builder().build::<_, ::hyper::Body>(https);
 
-    let credential = UsernamePasswordCredential::new("username".into(), "password".into());
-    let mut scheme = BasicAuthentication::new(&credential).into_scheme();
+    let credential = UsernamePasswordCredential::new("username", "password");
+    let mut scheme = BasicAuthentication::new(&credential);
 
     let mut status_codes = Vec::new();
 
@@ -55,7 +55,7 @@ async fn test_basic_authentication(
     Ok(())
 }
 
-/// Test basic authentication, passing the username and password in response to a 401 challenge.
+/// Basic authentication passing the username and password in response to a 401 challenge.
 #[::tokio::test]
 async fn test_basic_challenge() -> Result<(), Box<dyn std::error::Error + Send + Sync + 'static>> {
     let https = HttpsConnector::new();
@@ -64,10 +64,10 @@ async fn test_basic_challenge() -> Result<(), Box<dyn std::error::Error + Send +
     let mut realm_credentials = HashMap::new();
     realm_credentials.insert(
         "Fake Realm".into(),
-        UsernamePasswordCredential::new("username".into(), "password".into()),
+        UsernamePasswordCredential::new("username", "password"),
     );
-    let credential = HttpRealmCredential::new(realm_credentials);
-    let mut scheme = HttpAuthentication::new(&credential).into_scheme();
+    let credential = HttpRealmCredentials::new(realm_credentials);
+    let mut scheme = HttpAuthentication::new(&credential);
 
     let mut status_codes = Vec::new();
 
